SystemOrganization addCategory: #'SqueakNxt-Compiler-Programs'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Translation'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Api'!

!String methodsFor: '*SqueakNxt-Compiler' stamp: 'Richo 1/1/2011 15:42'!
asNxcRule
	^ NxcPrimitiveRule on: self ! !

Object subclass: #Nbc
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!Nbc commentStamp: 'Richo 1/2/2011 16:40' prior: 0!
A Nbc is a very simple API for accessing the Next Byte Codes Compiler using FFI on Windows. This class requires the actual nbc program installed in the default directory. You can download it from here "http://http://bricxcc.sourceforge.net/nbc/". FYI, I'm using the version 1.2.1.r3 (2010-06-29).

Examples: 
1) The simplest example would be compiling into a file called "output" and reporting errors to a file called "errors":

	Nbc new
		fileName: 'example_program.nbc';
		outFile: 'output';
		errorsFile: 'errors';
		execute.
		
2) To download a program into the Lego Nxt using the usb:

	Nbc new
		fileName: 'example_program.nbc';
		portName: 'usb';
		downloadProgram;
		execute.
		
3) To use the bluetooth on COM6 instead:
	
	Nbc new
		fileName: 'example_program.nbc';
		useBluetooth;
		portName: 'COM6';
		downloadProgram;
		execute.

Instance Variables
	parameters:		<WriteStream>

parameters
	- Used to build the parameters string to be sent to the nbc program. See "nbc -help" for more information.
!

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
beQuiet
	parameters space; nextPutAll: '-q'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
decompileProgram
	parameters space; nextPutAll: '-x'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
defineMacro: aByteString 
	parameters space; nextPutAll: '-D='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
dontCompile
	parameters space; nextPutAll: '-b'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
dontIncludeSystemFile
	parameters space; nextPutAll: '-n'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
downloadProgram
	parameters space; nextPutAll: '-d'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/28/2010 18:25'!
enhancedFirmware
	parameters space; nextPutAll: '-EF'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
errorsFile: aByteString 
	parameters space; nextPutAll: '-E='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'executing' stamp: 'Richo 12/29/2010 17:57'!
execute
	"Executes the command using FFI. Windows only for now"
	| result shell |
	shell := Smalltalk at: #Win32Shell ifAbsent: [^self error: 'FFI doesn''t seem to be installed'].
	(result := (shell new 
		shellExecute:  nil
		lpOperation:  'open'
		lpFile:  'nbc.exe'
		lpParameters:  parameters contents
		lpDirectory:  nil
		nShowCmd: 0) <= 32) ifTrue: [self error: 'system error, code:', result]! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
fileName: aByteString 
	parameters space; nextPutAll: aByteString! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
includePath: aByteString 
	parameters space; nextPutAll: '-I='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'initialize' stamp: 'Richo 12/25/2010 00:44'!
initialize
	super initialize.
	parameters := WriteStream on: ''! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/29/2010 17:59'!
nbc: fileName
	parameters space; nextPutAll: '-nbc='; nextPutAll: fileName! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 1/3/2011 14:07'!
optimizationLevel: number
	parameters space; nextPutAll: '-Z'; nextPutAll: number asString! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
outFile: aByteString 
	parameters space; nextPutAll: '-O='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
portName: aByteString 
	parameters space; nextPutAll: '-S='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:48'!
useBluetooth
	parameters space; nextPutAll: '-BT'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/28/2010 18:38'!
warningsOn
	parameters space; nextPutAll: '-w'! !

Object subclass: #NxcCompiler
	instanceVariableNames: 'programName source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!NxcCompiler commentStamp: 'Richo 12/31/2010 22:03' prior: 0!
A NxcCompiler is responsible for verifying/compiling programs as well as downloading them to the LegoNxt.

===============================
	"Code example"
	
	| program |
	
	"A simple nxc program"
	program := '
	#define MOVE_TIME   1000
	#define TURN_TIME    500

	task main()
	{
		repeat(4)
		{    
			OnFwd(OUT_AC, 75);
			Wait(MOVE_TIME);
			OnRev(OUT_C, 75);
		Wait(TURN_TIME);
		}  
		Off(OUT_AC);
	}'.
	
	"The code below will verify the program and return a collection of errors (if any)"
	NxcCompiler new
		source: program;
		verify.
	
	"The code below will download the program to the nxt using the usb cable. the resulting program will be called 'SimpleProgram'"
	NxcCompiler new
		programName: 'SimpleProgram';
		source: program;
		downloadUsingUsb.
===============================

Instance Variables
	programName:		<String>
	source:		<String>

programName
	- A name for the program. If not specified the program name will be the NxcCompiler instance's hash (not a nice name for a program, I must say).

source
	- The nxc source code of the program.
!

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 22:10'!
compileInFileNamed: outFile
	"Compiles the program into a file"
	^ self execute: (Nbc new outFile: outFile)! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:10'!
downloadUsingBluetooth: portName 
	"Compiles the program and downloads it using the bluetooth on the specified port name (COMn). Returns a collection of NbcErrors"
	^ self execute: (Nbc new portName: portName; useBluetooth; downloadProgram)! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:09'!
downloadUsingUsb
	"Compiles the program and downloads it using the usb cable. Returns a collection of NbcErrors"
	^ self execute: (Nbc new portName: 'usb'; downloadProgram)! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:14'!
errorsFileName
	"Temporary file name to store errors during compilation"
	^ self programName , '.err'! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 1/3/2011 14:07'!
execute: nbc 
	"Create the temporary files to store the source code and the errors.
	Execute nbc.
	Wait until nbc finishes and parse errors.
	Delete temporary files.
	Return errors"
	source isNil ifTrue: [^ self error: 'Source cannot be nil'].
	FileStream forceNewFileNamed: self programFileName do: [:file | file nextPutAll: source]; 
				forceNewFileNamed: self errorsFileName do: [:file | "Do nothing. Nbc will write this file"].
	[nbc fileName: self programFileName;
		 errorsFile: self errorsFileName;
		 optimizationLevel: 2;
		 execute.
	^ self parseErrors]
		ensure: [FileDirectory default deleteFileNamed: self programFileName;
				 deleteFileNamed: self errorsFileName]! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 1/3/2011 13:17'!
parse: errorsFile
"Returns a collection with all the errors founded by the nbc compiler. The errorsFile format is similar to this:

# Error: Unknown or invalid statement
File ""D:\RICHI\Squeak images\Nxt\simpleprogram.nbc"" ; line 7
#   result sbyt
#----------------------------------------------------------
1 error during compilation
"
| errors |
errors := OrderedCollection new.
[errorsFile peek = $#] 
	whileTrue: [ | errorDescription errorPosition errorSource |
		errorDescription := errorsFile nextLine allButFirst: 2.
		errorPosition := (errorsFile nextLine subStrings: {$;}) last withoutLeadingBlanks.
		errorSource := errorsFile nextLine allButFirst: 4.
		errorsFile nextLine.
		errors add: (NxcError description: errorDescription position: errorPosition source: errorSource).
		].
^errors asArray! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 22:05'!
parseErrors
	"Wait until the errors file is fully written by nbc, then read it and return the errors.
	IMPORTANT: Be aware that this code relies on the fact that FileStream>>#fileNamed: returns nil when the file is being modified by another process. If this somehow changes in the future, this code will inevitably break."
	| errorsFile |
	[500 milliSeconds asDelay wait.
	(errorsFile := FileStream fileNamed: self errorsFileName) isNil] whileTrue.
	[^ self parse: errorsFile]
		ensure: [errorsFile close].
! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 1/1/2011 03:07'!
program: aNxcProgram
	source := aNxcProgram source.
	programName isNil ifTrue: [programName := aNxcProgram name]! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/29/2010 17:43'!
programFileName
	"Temporary file name to store the source code"
	^ self programName , '.nxc'! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 22:10'!
programName
	"If programName is not specified, the default name will be this instance's hash"
	^ programName ifNil: [self hash asString]! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:47'!
programName: aString
	programName := aString! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 17:07'!
source
	^ source! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:48'!
source: aString
	source := aString.! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 22:10'!
verify
	"Compiles the program and returns a collection of NbcErrors. If the collection is empty then the program is ok"
	^ self execute: Nbc new! !

Object subclass: #NxcError
	instanceVariableNames: 'description position source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!NxcError class methodsFor: 'instance creation' stamp: 'Richo 12/25/2010 16:02'!
description: description position: position source: source
^ (self basicNew setDescription: description position: position source: source) initialize ! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
description
	^ description ! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
position
	^ position ! !

!NxcError methodsFor: 'printing' stamp: 'Richo 12/27/2010 22:08'!
printOn: aStream
aStream nextPutAll: position asString;
		 nextPut: $:;
		 space;
		 nextPutAll: description asString 
! !

!NxcError methodsFor: 'private' stamp: 'Richo 12/25/2010 16:02'!
setDescription: aByteString position: aByteString2 source: aByteString3 
	description := aByteString.
	position := aByteString2.
	source := aByteString3! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
source
	^ source ! !

Object subclass: #NxcProgram
	instanceVariableNames: 'name source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

!NxcProgram commentStamp: 'Richo 1/2/2011 00:00' prior: 0!
A NxcProgram represents a program written in nxc language. A NxcProgram can be created by sending #asNxcProgram to a NxtProgram subclass. This will create an instance of NxcProgram, which understands a couple of useful methods that make compiling/downloading very easy. This methods are #verify, #compileInFileNamed:, #downloadUsingBluetooth:, and #downloadUsingUsb.
For example, the line below will compile and download the program using its default name ("NxtProgramTest2").

	NxtProgramTest2 asNbcProgram downloadUsingUsb.

To specify a different name, you can use:

	(NxtProgramTest2 asNbcProgram: 'anotherName') downloadUsingUsb.

Instance Variables
	name:		<String>
	source:		<String>

name
	- The program's name.

source
	- The program's source in nxc.
!

!NxcProgram class methodsFor: 'instance creation' stamp: 'Richo 1/1/2011 03:01'!
name: programName source: programSource
	^ (self basicNew setName: programName source: programSource) initialize ! !

!NxcProgram methodsFor: 'compiling' stamp: 'Richo 1/1/2011 15:33'!
compileInFileNamed: fileName
	^ NxcCompiler new program: self; compileInFileNamed: fileName! !

!NxcProgram methodsFor: 'compiling' stamp: 'Richo 1/1/2011 15:33'!
downloadUsingBluetooth: portName
	^ NxcCompiler new program: self; downloadUsingBluetooth: portName ! !

!NxcProgram methodsFor: 'compiling' stamp: 'Richo 1/1/2011 15:33'!
downloadUsingUsb
	^ NxcCompiler new program: self; downloadUsingUsb! !

!NxcProgram methodsFor: 'accessing' stamp: 'Richo 1/1/2011 03:07'!
name
	^ name! !

!NxcProgram methodsFor: 'printing' stamp: 'Richo 1/1/2011 21:51'!
printOn: aStream
	aStream nextPutAll: source! !

!NxcProgram methodsFor: 'private' stamp: 'Richo 1/1/2011 03:02'!
setName: aByteSymbol source: aWriteStream 
	name := aByteSymbol.
	source := aWriteStream contents.! !

!NxcProgram methodsFor: 'accessing' stamp: 'Richo 1/1/2011 03:07'!
source
	^ source! !

!NxcProgram methodsFor: 'compiling' stamp: 'Richo 1/1/2011 03:02'!
verify
	^ NxcCompiler new program: self; verify! !

Object subclass: #NxtProgram
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgram commentStamp: 'Richo 1/2/2011 00:11' prior: 0!
A NxtProgram is an abstract class whose subclasses can be translated to the nxc language and be downloaded to a Lego Nxt robot. It only has some methods defined for documentation purposes, but they're not actually needed.

The class side is more interesting because:
	* it has a few methods that make translating more easy.
	* its class side method #definition has been changed to show an extra option that allows to specify the type of the inst vars (it will only appear for subclasses, though).!

NxtProgram subclass: #DodgeWall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!DodgeWall methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 18:06'!
main
	self setSensorLowSpeed: 3.
	[
	(self sensorUS: 3) < 30
		ifTrue: [self onFwd: 0 speed: 100.
	self onFwd: 2 speed: -100]
		ifFalse: [self onFwd: 0 speed: 100.
	self onFwd: 2 speed: 100]
	] repeat.! !

NxtProgram subclass: #FollowLineExample
	instanceVariableNames: 'black'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!FollowLineExample class methodsFor: 'definition' stamp: 'Richo 1/2/2011 22:22'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'int' withBlanksCondensed! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 14:25'!
isOnBlack
	<returnType: 'bool'>
	^ (self sensorValue: 0) < (black + 10)! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 14:25'!
isOnWhite
	<returnType: 'bool'>
	^ self isOnBlack not! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 14:26'!
main
	"Configure sensor"
	self setSensorLight: 0 active: true.
	
	"Get 100 samples in a second and use average as black value"
	black := 0.
	100 timesRepeat: [black := black + (self sensorValue: 0). self wait: 10].
	black := black / 100.
	
	[true] whileTrue: [
		self turnLeft.
		[self isOnWhite] whileTrue. "Wait until black"
		[self isOnBlack] whileTrue. "Wait until white"
		self turnRight.
		[self isOnWhite] whileTrue. "Wait until black"
		[self isOnBlack] whileTrue. "Wait until white"
	]! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:29'!
turnLeft
	self onFwd: 0 speed: 100;
		onFwd: 2 speed: 25! !

!FollowLineExample methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:29'!
turnRight
	self onFwd: 2 speed: 100;
		onFwd: 0 speed: 25! !

NxtProgram subclass: #MultithreadingExample1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!MultithreadingExample1 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 18:29'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!MultithreadingExample1 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 18:41'!
main
[[self onFwd: 0 speed: 100] repeat] forkNamed: 'Der'.
[[self onFwd: 2 speed: -100] repeat] forkNamed: 'Izq'.! !

NxtProgram subclass: #MultithreadingExample2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!MultithreadingExample2 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 18:50'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!MultithreadingExample2 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 18:53'!
main
[self onFwd: 0 speed: 100. self wait: 1000. self off: 0] fork.
[[self onFwd: 2 speed: -100] repeat] fork.! !

NxtProgram subclass: #NxcTutorialExample1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxcTutorialExample1 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 15:45'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxcTutorialExample1 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 15:46'!
main
	self onFwd: 0 speed: 75;
		 onFwd: 2 speed: 75.
	self wait: 4000.
	self onRev: 0 speed: 75;
		 onRev: 2 speed: 75.
	self wait: 4000.
	self off: 0; off: 2! !

NxtProgram subclass: #NxcTutorialExample2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxcTutorialExample2 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 16:19'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxcTutorialExample2 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:42'!
main
	| moveTime turnTime |
	<tempTypes: 'int int'>
	moveTime := 1000.
	turnTime := 500.
	10
		timesRepeat: [4
				timesRepeat: [self onFwd: 0 speed: 75;
						 onFwd: 2 speed: 75.
					self wait: moveTime.
					self onRev: 2 speed: 75.
					self wait: turnTime]].
	self off: 0; 
		 off: 2! !

NxtProgram subclass: #NxcTutorialExample3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxcTutorialExample3 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 16:44'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxcTutorialExample3 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:45'!
main
	
	| moveTime turnTime |
	turnTime := 360.
	moveTime := 200.
	50 timesRepeat: [
		self onFwd: 0 speed: 75; onFwd: 2 speed: 75.
		self wait: moveTime.
		self onRev: 2 speed: 75.
		self wait: turnTime.
		moveTime := moveTime + 200.
		].
	self off: 0; off: 2! !

!NxtProgram class methodsFor: 'translating' stamp: 'Richo 1/2/2011 00:02'!
asNxcProgram
	"Useful method to easily translate a program to nxc. By default the class name is used as program name"
	^ self asNxcProgram: self name! !

!NxtProgram class methodsFor: 'translating' stamp: 'Richo 1/2/2011 00:02'!
asNxcProgram: programName 
	"Useful method to easily translate a program to nxc"
	^ NxcTranslator new translate: self asProgramNamed: programName! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:38'!
definition
	"Answer a String that defines the receiver."
	"Overwritten to show the #instanceVariablesTypes: message in the class definition of NxtProgram subclasses"
	| aStream path |
	self = NxtProgram ifTrue: [^super definition].
	aStream := WriteStream
				on: (String new: 300).
	superclass == nil
		ifTrue: [aStream nextPutAll: 'ProtoObject']
		ifFalse: [path := ''.
			self environment
				scopeFor: superclass name
				from: nil
				envtAndPathIfFound: [:envt :remotePath | path := remotePath].
			aStream nextPutAll: path , superclass name].
	aStream nextPutAll: self kindOfSubclass;
		 store: self name.
	[(self hasTraitComposition
			and: [self traitComposition notEmpty])
		ifTrue: [aStream cr; tab; nextPutAll: 'uses: '; nextPutAll: self traitCompositionString]] on: MessageNotUnderstood do: ["Ignore DNU. Traits don't exist in Etoys"].
	aStream cr; tab; nextPutAll: 'instanceVariableNames: '; store: self instanceVariablesString.
	aStream cr; tab; nextPutAll: 'instanceVariableTypes: '; store: self instanceVariableTypesString.
	aStream cr; tab; nextPutAll: 'classVariableNames: '; store: self classVariablesString.
	aStream cr; tab; nextPutAll: 'poolDictionaries: '; store: self sharedPoolsString.
	aStream cr; tab; nextPutAll: 'category: '; store: (SystemOrganization categoryOfElement: self name) asString.
	superclass
		ifNil: [aStream nextPutAll: '.';
				 cr.
			aStream nextPutAll: self name.
			aStream space; nextPutAll: 'superclass: nil'].
	^ aStream contents! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:38'!
definitionST80
	"Only useful for the Etoys image"
	self = NxtProgram ifTrue: [^super definitionST80].
	^ self definition! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:51'!
instVarTypes 
	"Returns a collection of symbols representing the nxc type of each inst var in the order ther are defined"
	| instVarTypes |
	instVarTypes := (self instanceVariableTypesString subStrings: {Character space}) collect: [:each | each asSymbol].
	instVarTypes isEmpty ifTrue: [instVarTypes := #()].
	^ instVarTypes ! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:29'!
instanceVariableTypes: aString 
"Automatically compiles a method called #instanceVariableTypesString that returns the string given as a parameter"
self class compile: 'instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you''re doing"
^ ', aString printString, ' withBlanksCondensed' classified: #definition
! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:24'!
instanceVariableTypesString
	"This method will be automaticaly generated for subclasses whenever a subclass of NxtProgram is compiled. It returns the information of the inst vars' type. I choose to do it this way (instead of using an class inst var) to ensure the type info will be saved when we file out this class or store it using Monticello"
	^ ''! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:25'!
subclass: nameOfClass instanceVariableNames: instVarNames instanceVariableTypes: instVarTypes classVariableNames: classVarNames poolDictionaries: poolDictnames category: category
"Overwritten to allow defining the inst vars' type info of a class"
	^(ClassBuilder new
		superclass: self
		subclass: nameOfClass
		instanceVariableNames: instVarNames
		classVariableNames: classVarNames
		poolDictionaries: poolDictnames
		category: category) instanceVariableTypes: instVarTypes! !

!NxtProgram methodsFor: 'display module' stamp: 'Richo 1/2/2011 18:01'!
displayNumber: value x: x y: y
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

char NumOut  ( int  x,  
  int  y,  
  variant  value,  
  unsigned long  options = DRAW_OPT_NORMAL   
 )   [inline] 

Draw a number. 

Draw a numeric value on the screen at the specified x and y location. The y value must be a multiple of 8. Valid line number constants are listed in the Line number constants group. Optionally specify drawing options. If this argument is not specified it defaults to DRAW_OPT_NORMAL. Valid display option constants are listed in the Drawing option constants group. 

See also:
SysDrawText, DrawTextType
Parameters:
 x  The x value for the start of the number output.  
 y  The text line number for the number output.  
 value  The value to output to the LCD screen. Any numeric type is supported.  
 options  The optional drawing options.  

Returns:
The result of the drawing operation. 

-------------------------------------------------------------------------------------
Note from Richo: The options parameter is not supported, use #displayNumber:x:y:options: for that."
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'display module' stamp: 'Richo 1/2/2011 18:01'!
displayNumber: value x: x y: y options: options
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

char NumOut  ( int  x,  
  int  y,  
  variant  value,  
  unsigned long  options = DRAW_OPT_NORMAL   
 )   [inline] 

Draw a number. 

Draw a numeric value on the screen at the specified x and y location. The y value must be a multiple of 8. Valid line number constants are listed in the Line number constants group. Optionally specify drawing options. If this argument is not specified it defaults to DRAW_OPT_NORMAL. Valid display option constants are listed in the Drawing option constants group. 

See also:
SysDrawText, DrawTextType
Parameters:
 x  The x value for the start of the number output.  
 y  The text line number for the number output.  
 value  The value to output to the LCD screen. Any numeric type is supported.  
 options  The optional drawing options.  

Returns:
The result of the drawing operation. 

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'display module' stamp: 'Richo 1/4/2011 10:21'!
displayText: value x: x y: y
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/4/2011 10:19'!
float: outputPort
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/3/2011 15:46'!
off: outputPort
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/2/2011 18:21'!
onFwd: outputs speed: speed
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

void OnFwd  ( byte  outputs,  
  char  pwr   
 )   [inline] 

Run motors forward. 

Set outputs to forward direction and turn them on.

Parameters:
 outputs  Desired output ports. Can be a constant or a variable, see Output port constants. If you use a variable and want to control multiple outputs in a single call you need to use a byte array rather than a byte and store the output port values in the byte array before passing it into this function.  
 pwr  Output power, 0 to 100. Can be negative to reverse direction.  

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/2/2011 18:21'!
onRev: output speed: speed
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

void OnRev  ( byte  outputs,  
  char  pwr   
 )   [inline] 

Run motors backward. 

Set outputs to reverse direction and turn them on.

Parameters:
 outputs  Desired output ports. Can be a constant or a variable, see Output port constants. If you use a variable and want to control multiple outputs in a single call you need to use a byte array rather than a byte and store the output port values in the byte array before passing it into this function.  
 pwr  Output power, 0 to 100. Can be negative to reverse direction.  

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'sound module' stamp: 'Richo 1/4/2011 10:22'!
playFile: fileName
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'sound module' stamp: 'Richo 1/4/2011 10:22'!
playTone: freq duration: milliseconds
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'output module' stamp: 'Richo 1/4/2011 10:20'!
rotateMotor: outputPort speed: speed angle: angle
"NXC Programmer's guide -  October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
sensorBoolean: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
sensorNormalized: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
sensorRaw: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
sensorScaled: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:23'!
sensorUS: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/2/2011 22:06'!
sensorValue: port
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

unsigned int SensorValue  ( const byte &  port  )  [inline] 

Read sensor scaled value. 

Return the processed sensor reading for a sensor on the specified port. This is the same value that is returned by the sensor value names (e.g. SENSOR_1) or the Sensor function.

Parameters:
 port  The sensor port. See Input port constants. A variable whose value is the desired sensor port may also be used.  

Returns:
The sensor's scaled value. 
-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/2/2011 22:06'!
setSensorLight: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

void SetSensorLight  ( const byte &  port,  
  bool  bActive = true   
 )   [inline] 

Configure a light sensor. 

Configure the sensor on the specified port as an NXT light sensor. 

Parameters:
 port  The port to configure. See Input port constants.  
 bActive  A boolean flag indicating whether to configure the port as an active or inactive light sensor. The default value for this optional parameter is true.  
 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/2/2011 22:06'!
setSensorLight: inputPort active: aBoolean
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 void SetSensorLight  ( const byte &  port,  
  bool  bActive = true   
 )   [inline] 

Configure a light sensor. 

Configure the sensor on the specified port as an NXT light sensor. 

Parameters:
 port  The port to configure. See Input port constants.  
 bActive  A boolean flag indicating whether to configure the port as an active or inactive light sensor. The default value for this optional parameter is true.  
-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
setSensorLowSpeed: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:19'!
setSensorLowSpeed: inputPort scaling: aBoolean
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:18'!
setSensorSound: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:18'!
setSensorSound: inputPort scaling: aBoolean
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'input module' stamp: 'Richo 1/4/2011 10:18'!
setSensorTouch: inputPort
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

 -------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

!NxtProgram methodsFor: 'command module' stamp: 'Richo 1/2/2011 18:22'!
wait: ms
"NXC Programmer's guide - October 6, 2010 - by John Hansen
-------------------------------------------------------------------------------------

void Wait  ( unsigned long  ms  )  [inline] 

Wait some milliseconds. 

Make a task sleep for specified amount of time (in 1000ths of a second).

Parameters:
 ms  The number of milliseconds to sleep.  

-------------------------------------------------------------------------------------"
self error: 'Just a comment'! !

NxtProgram subclass: #NxtProgramTest1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest1 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:22'!
anotherMethod
	self displayNumber: 100 x: 10 y: 50 - (10*5) ! !

!NxtProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 12/29/2010 14:23'!
main
	self onFwd: 0 speed: 100;
		wait: 1000.! !

NxtProgram subclass: #NxtProgramTest2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest2 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest2 methodsFor: 'as yet unclassified' stamp: 'Richo 12/29/2010 14:19'!
main
	| var |
	var := 10 + 20 + (30 * 40).
	self displayNumber: var x: 10 y: 8;
		wait: var;
		onFwd: 0 speed: 100! !

NxtProgram subclass: #NxtProgramTest3
	instanceVariableNames: 'instVar1 instVar2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest3 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'byte   char' withBlanksCondensed! !

!NxtProgramTest3 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:59'!
func: arg1 and: arg2
	<argTypes: 'long long'>
	<tempTypes: 'long long'>
	<returnType: 'byte'>
	| temp1 instVar2 |
	instVar2 := instVar1 .
	temp1 := instVar1.
	instVar1 := temp1 * instVar2.
	^ arg1 + arg2 
! !

!NxtProgramTest3 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:59'!
main
	<tempTypes: 'long'>
	| tempVar1 |
	tempVar1 := self func: 3 and: instVar1.
	instVar2 := 4.
	self displayNumber: tempVar1 * instVar2 x: 0 y: 0! !

NxtProgram subclass: #NxtProgramTest4
	instanceVariableNames: 'instVar1 instVar2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest4 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'long long' withBlanksCondensed! !

!NxtProgramTest4 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:55'!
func: arg1 and: arg2
	<argTypes: 'long'>
	<returnType: 'int'>
	| temp1 instVar2 |
	instVar2 := instVar1 .
	temp1 := instVar1.
	instVar1 := temp1 * instVar2.
	^ arg1 + arg2 
! !

!NxtProgramTest4 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:56'!
main
	<tempTypes: 'long'>
	| tempVar1 |
	tempVar1 := self func: 3 and: instVar1.
	instVar2 := 4.
	self displayNumber: tempVar1 * instVar2 x: 0 y: 0! !

NxtProgram subclass: #NxtProgramTest5
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest5 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest5 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 14:07'!
main
	
	5 = true
		ifTrue: [self wait: 1000.
			self displayNumber: 500 x: 0 y: 0]
		ifFalse: [self wait: 1200; displayNumber: 500 x: 0 y: 0].
		
	false ifTrue: [self onFwd: 0 speed: 100].
	true ifFalse: [self onFwd: 0 speed: -100]! !

NxtProgram subclass: #NxtProgramTest6
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest6 class methodsFor: 'definition' stamp: 'Richo 1/2/2011 23:56'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest6 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 00:20'!
main
	<tempTypes: 'int'>
	| i |
	i := 1.
	20 timesRepeat: [
	self displayNumber: i x:100  y: 100;
		wait: 1000.
	i := i + 1.
	]! !

NxtProgram subclass: #NxtProgramTest7
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest7 class methodsFor: 'definition' stamp: 'Richo 1/3/2011 16:34'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest7 methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 16:39'!
main
	| var |
	var := 1000.
	10 timesRepeat: [
	10 timesRepeat: [
		4 timesRepeat: [
			self onFwd: 0 speed: 100;
				 wait: var.
			]
		].
	]! !

Object subclass: #VariableInfo
	instanceVariableNames: 'variables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

!VariableInfo commentStamp: 'Richo 1/1/2011 20:53' prior: 0!
A VariableInfo stores the info (scope, name and type) of all the variables found in a program.

Instance Variables
	variables:		<Dictionary>

variables
	- This dictionary uses the method selectors as keys (and a special key called #'$instVars$' to store variables with a global scope). For each key it stores a set of associations, each association's key being the variable's name and value being its type.!

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 12/31/2010 23:51'!
at: selector
	(variables includesKey: selector) ifFalse: [variables at: selector put: Set new].
	^ variables at: selector ! !

!VariableInfo methodsFor: 'initialize-release' stamp: 'Richo 12/31/2010 22:21'!
initialize
	super initialize.
	variables := Dictionary new! !

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 1/1/2011 21:47'!
instVars
	"I use this weird symbol to avoid a name clash with a method selector"
	^ self at: #'$instVars$'! !

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 1/1/2011 21:47'!
isAnInstVar: varName
	"Looks for a variable with the same name in the instVars set"
	^ (self instVars detect: [:each | each name = varName] ifNone: [nil]) notNil ! !

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 1/1/2011 02:58'!
typeOf: var in: method
	^ ((self at: method) detect: [:each | each key = var]) value! !

RBProgramNodeVisitor subclass: #NxcTaskExtractor
	instanceVariableNames: 'tasks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

!NxcTaskExtractor methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 18:52'!
acceptMessageNode: aMessageNode
	aMessageNode selector caseOf: {
		[#fork] -> [tasks add: aMessageNode receiver -> self newTaskName].
		[#forkNamed:] -> [tasks add: aMessageNode receiver -> aMessageNode arguments first value].
	} otherwise: []! !

!NxcTaskExtractor methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 18:37'!
extractTasksFrom: aClass 
	| methods |
	methods := aClass selectors
				collect: [:each | aClass parseTreeFor: each].
	tasks := Set new.
	methods
		do: [:each | self visitNode: each].
	^ tasks! !

!NxcTaskExtractor methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 22:14'!
newTaskName
	^ 'task', self hash asString, tasks size asString! !

RBProgramNodeVisitor subclass: #NxcTranslator
	instanceVariableNames: 'variableInfo source tasks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

!NxcTranslator commentStamp: 'Richo 1/2/2011 17:37' prior: 0!
A NxcTranslator is responsible for translating a subclass of NxtProgram from smalltalk to Nxc. It's public protocol is very simple, just use #translate:asProgramNamed:, which returns an instance of NxcProgram.
An even easier way of translating programs is sending #asNxcProgram to a NxtProgram subclass. For example:

	NxtProgramTest1 asNxcProgram.
	
You can also specify a different program name:

	NxtProgramTest1 asNxcProgram: 'Other program name'.
	
A NxcTranslator is a subclass of RBProgramNodeVisitor and it uses the visitor pattern to navigate through the parse tree translating every node into a stream called source. 
The translation of MessageNodes is done via NxcPrimitiveRules. In particular, see NxcPrimitiveRule class>>#initialize which creates a Dictionary with all the defined rules.


Instance Variables
	source:		<Stream>
	variableInfo:		<VariableInfo>

source
	- The stream in which the source will be translated.

variableInfo
	- Contains the info of the variables found in the class (inst vars, temp vars and arguments).
!

NxcTranslator subclass: #NxcPrimitiveRule
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

NxcPrimitiveRule class
	instanceVariableNames: 'rules'!

!NxcPrimitiveRule commentStamp: 'Richo 1/2/2011 17:37' prior: 0!
A NxcPrimitiveRule is is a very simple way of defining a rule to translate a smalltalk message to a nxc primitive. Every function (such as "NumOut" or "Wait"), every operator (such as "+" or "=="), and every control structure (such as "if", "while", or "repeat") is considered to be a nxc primitive. 
To create a rule simply send #asNxcRule to a proper string. For example:

	'NumOut(arg2, arg3, arg1)' asNxcRule.
	'(arg0 + arg1)' asNxcRule.
	'if(arg0){arg1}else{arg2}' asNxcRule.	
	
Once a rule is created it can be applied to a message node by sending #applyTo:. This mehod will then visit the message node receiver and arguments and replace the 'arg#' in the string with the result (arg0 represents the receiver). Making it a subclass of NxcTranslator ensures we translate the nodes just as if they were visited by the actual translator and avoids repeating code.
To see all the defined rules look at NxcPrimitiveRule class>>#initialize.

Instance Variables
	value:		<String>

value
	- The string that defines how to apply the rule.
!

NxcPrimitiveRule class
	instanceVariableNames: 'rules'!

!NxcPrimitiveRule class methodsFor: 'rules' stamp: 'Richo 1/2/2011 17:36'!
allRules
	^ rules values collect: [:each | each asNxcRule]! !

!NxcPrimitiveRule class methodsFor: 'rules' stamp: 'Richo 1/4/2011 10:15'!
allRulesSelectors
	^ rules keys! !

!NxcPrimitiveRule class methodsFor: 'class initialization' stamp: 'Richo 1/4/2011 10:23'!
initialize
	"This method defines all the rules for translating nxc primitives. Rules is a Dictionary where each key is a Smalltalk selector and each value is a string which can be converted to a NxcPrimitiveRule by sending the #asNxcRule message."
	"
	self initialize.
	"
	rules := {
	
		"Input"
		#setSensorLight: -> 'SetSensorLight(arg1)'.
		#setSensorLight:active: -> 'SetSensorLight(arg1, arg2)'.
		#setSensorTouch: -> 'SetSensorTouch(arg1)'.
		#setSensorSound: -> 'SetSensorSound(arg1)'.
		#setSensorSound:scaling: -> 'SetSensorSound(arg1, arg2)'.
		#setSensorLowSpeed: -> 'SetSensorLowspeed(arg1)'.
		#setSensorLowSpeed:isPowered: -> 'SetSensorLowspeed(arg1, arg2)'.
		#sensorValue: -> 'SensorValue(arg1)'.
		#sensorBoolean: -> 'SensorBoolean(arg1)'.
		#sensorNormalized: -> 'SensorNormalized(arg1)'.
		#sensorRaw: -> 'SensorRaw(arg1)'.
		#sensorScaled: -> 'SensorScaled(arg1)'.
		#sensorUS: -> 'SensorUS(arg1)'.
		
		"Output"
		#onFwd:speed: -> 'OnFwd(arg1, arg2)'.
		#onRev:speed: -> 'OnRev(arg1, arg2)'.
		#off: -> 'Off(arg1)'.
		#float: -> 'Float(arg1)'.
		#rotateMotor:speed:angle: -> 'RotateMotor(arg1, arg2, arg3)'.

		"Display"
		#displayNumber:x:y: -> 'NumOut(arg2, arg3, arg1)'.
		#displayNumber:x:y:options: -> 'NumOut(arg2, arg3, arg1, arg4)'.
		#displayText:x:y: -> 'TextOut(arg2, arg3, arg1)'.
						
		"Sound"
		#playFile: -> 'PlayFile(arg1)'.
		#playTone:duration: -> 'PlayTone(arg1, arg2)'.		
			
		"Command"
		#wait: -> 'Wait(arg1)'.
	
		"Operators"
		#+ -> '(arg0 + arg1)'.
		#- -> '(arg0 - arg1)'.
		#* -> '(arg0 * arg1)'.
		#/ -> '(arg0 / arg1)'.
		#= -> '(arg0 == arg1)'.
		#> -> '(arg0 > arg1)'.
		#< -> '(arg0 < arg1)'.
		#not -> '!!(arg0)'.
		
		"Math"
		#sqrt -> 'Sqrt(arg0)'.
		#sin -> 'Sin(arg0)'.
		#cos -> 'Cos(arg0)'.
		#arcSin -> 'Asin(arg0)'.
		#arcCos -> 'Acos(arg0)'.
		#arcTan -> 'Atan(arg0)'.
		#ceiling -> 'Ceil(arg0)'.
		#exp -> 'Exp(arg0)'.
		#floor -> 'Floor(arg0)'.
		#tan -> 'Tan(arg0)'.
		#ln -> 'Log(arg0)'.
		#log -> 'Log10(arg0)'.
		#raisedTo: -> 'Pow(arg0, arg1)'.
		#truncated -> 'Trunc(arg0)'.
		#degreeSin -> 'SinD(arg0)'.
		#degreeCos -> 'CosD(arg0)'.
		#degreeTan -> 'TanD(arg0)'.
		#degreeArcTan -> 'ATanD(arg0)'.
						
		"Control structures"
		#ifTrue:ifFalse: -> 'if(arg0){arg1}else{arg2}'.
		#ifTrue: -> 'if(arg0){arg1}'.
		#ifFalse: -> 'if(arg0){}else{arg1}'.
		#whileTrue -> 'while(arg0)'.
		#whileTrue: -> 'while(arg0){arg1}'.
		#timesRepeat: -> 'repeat(arg0){arg1}'.
		#doWhileTrue: -> 'do{arg0}while(arg1)'.
		#repeat -> 'while(true){arg0}'.
		
	} as: Dictionary! !

!NxcPrimitiveRule class methodsFor: 'rules' stamp: 'Richo 1/2/2011 17:32'!
isNxcPrimitive: selector
	^ rules includesKey: selector! !

!NxcPrimitiveRule class methodsFor: 'instance creation' stamp: 'Richo 1/1/2011 13:00'!
on: aByteString 
	^ (self basicNew setValue: aByteString) initialize ! !

!NxcPrimitiveRule class methodsFor: 'rules' stamp: 'Richo 1/2/2011 17:39'!
ruleFor: selector
	^ (rules at: selector) asNxcRule ! !

!NxcPrimitiveRule methodsFor: 'applying' stamp: 'Richo 1/3/2011 00:16'!
applyTo: aMessageNode 
	"Visits the message node receiver and arguments and replaces the 'arg*' in the string with the result (arg0 represents the receiver). It reinitializes the source stream for every node so that we only get the result of visiting that node"
	| rule |
	rule := value.
	({aMessageNode receiver}, aMessageNode arguments)
		doWithIndex: [:node :index | | token |
			token := 'arg', (index - 1) asString.
			source := WriteStream on: ''.
			self visitNode: node;
				 removeSemicolonIfNecessary: token.
			rule := rule copyReplaceAll: token with: source contents
			].
	^ rule! !

!NxcPrimitiveRule methodsFor: 'printing' stamp: 'Richo 1/2/2011 17:35'!
printOn: aStream
super printOn: aStream.
aStream nextPut: $(.
aStream nextPutAll: value.
aStream nextPut: $)! !

!NxcPrimitiveRule methodsFor: 'applying' stamp: 'Richo 1/3/2011 00:18'!
removeSemicolonIfNecessary: token
	"Depending on the position of the token in the rule string, it can be necessary to remove the semicolon after the statement. For now, we only leave the semicolon if the token is surrounded by curly brackets {}.
	For instance, in the rule below
	
	'while(arg0){arg1}'
	
	The node replacing the arg0 token will have its semicolon removed, whereas the arg1 won't. Of course, most of the time it won't be necessary to remove any semicolon (if arg0 is a literal or a variable, for instance)."
	| index newContents oldContents rule |
	oldContents := source contents.
	rule := value withBlanksCondensed.
	
	"Do nothing if the source does not end with a semicolon"
	(oldContents size > 2 and: [oldContents endsWith: ';', String cr]) ifFalse: [^self].

	index := rule findString: token.

	"Do nothing if the token is not in the rule"
	index = 0 ifTrue: [^self].

	"Do nothing if the token is not surrounded by curly brackets"
	((rule at: index - 1) = ${ and: [(rule at: index + token size) = $}]) ifTrue: [^self].

	"Okay, you can now safely remove the semicolon"
	newContents := oldContents allButLast: 2.
	source reset; nextPutAll: newContents.! !

!NxcPrimitiveRule methodsFor: 'private' stamp: 'Richo 1/1/2011 13:00'!
setValue: aByteString 
	value := aByteString ! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/3/2011 16:42'!
acceptAssignmentNode: anAssignmentNode
	self visitNode: anAssignmentNode variable.
	source space; nextPut: $=; space.
	self visitNode: anAssignmentNode value.
	source nextPut: $;; cr.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/2/2011 20:48'!
acceptCascadeNode: aCascadeNode
	aCascadeNode messages do: [:each | self visitNode: each. source nextPut: $;; cr].! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 03:53'!
acceptLiteralNode: aLiteralNode
	source nextPutAll: aLiteralNode value asString.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/3/2011 18:55'!
acceptMessageNode: aMessageNode 
	(aMessageNode selector = #forkNamed: or: [aMessageNode selector = #fork])
		ifTrue: [| taskName |
			taskName := (tasks
						detect: [:each | each key = aMessageNode receiver]) value.
			source nextPutAll: 'StartTask(' , taskName , ')'.
			^ self].
	(NxcPrimitiveRule isNxcPrimitive: aMessageNode selector)
		ifTrue: [source
				nextPutAll: ((NxcPrimitiveRule ruleFor: aMessageNode selector)
						applyTo: aMessageNode).
			^ self].
	source
		nextPutAll: (aMessageNode selector asIdentifier: false);
		 nextPut: $(.
	aMessageNode arguments
		do: [:each | self visitNode: each]
		separatedBy: [source nextPut: $,;
				 space].
	source nextPut: $)! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/3/2011 00:14'!
acceptMethodNode: methodNode
	self storeMethodDeclaration: methodNode.
	source cr; nextPut: ${; cr.
	methodNode allTemporaryVariables do: [:var |  source nextPutAll: (variableInfo typeOf: var in: methodNode selector); space; nextPutAll: var; nextPut: $;; cr].
	self visitNode: methodNode body.
	source nextPut: $}.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 13:28'!
acceptPragmaNode: aPragmaNode
	"Do nothing"! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/2/2011 19:53'!
acceptReturnNode: aReturnNode
	source nextPutAll: 'return'; space.
	self visitNode: aReturnNode value.
	source nextPut: $;; cr.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/2/2011 21:02'!
acceptSequenceNode: aSequenceNode
aSequenceNode statements do: [:each | self visitNode: each. 
	each isMessage ifTrue: [source nextPut: $;; cr]].! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 03:28'!
acceptVariableNode: aVariableNode
	source nextPutAll: aVariableNode name! !

!NxcTranslator methodsFor: 'type info' stamp: 'Richo 1/3/2011 15:53'!
defaultVariableType
	"If a variable type is not defined it will use this"
	^ #int! !

!NxcTranslator methodsFor: 'type info' stamp: 'Richo 1/1/2011 21:49'!
extractVariablesFrom: aClass
"This method populates the variableInfo. First it adds all the instVars, then it goes over every selector and adds all the arguments and temporaries. To know the type of the variables it checks pragma definitions"
variableInfo := VariableInfo new.
aClass instVarNames doWithIndex: [:varName :index |
	variableInfo instVars add: (varName -> (aClass instVarTypes at: index ifAbsent: [self defaultVariableType]))
	].
aClass selectors do: [:selector | | methodNode types |

	methodNode := (aClass parseTreeFor: selector).
	types := self getTypes: #argTypes: from: methodNode.
	methodNode allArgumentVariables doWithIndex: [:varName :index |
		(variableInfo at: selector) add: varName -> (types at: index ifAbsent: [self defaultVariableType])
		].
	types := self getTypes: #tempTypes: from: methodNode.
	methodNode allTemporaryVariables doWithIndex: [:varName :index |
		(variableInfo at: selector) add: varName -> (types at: index ifAbsent: [self defaultVariableType])
		].
	].
^ variableInfo
! !

!NxcTranslator methodsFor: 'type info' stamp: 'Richo 1/1/2011 21:50'!
getTypes: aSymbol from: aMethodNode 
"It checks the pragmas of the method node to find type definitions"
| pragma |
pragma := aMethodNode pragmas detect: [:each | each selector = aSymbol] ifNone: [^#()].
^ (pragma arguments first value subStrings: {Character space}) collect: [:each | each asSymbol]! !

!NxcTranslator methodsFor: 'type info' stamp: 'Richo 1/1/2011 21:50'!
returnTypeOf: aMethodNode 
	"It looks for the #returnType: pragma to get the return type of the method"
	| pragma |
	pragma := aMethodNode pragmas detect: [:each | each selector = #returnType:] ifNone: [^#void].
	^ pragma arguments first value asSymbol.
	! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/3/2011 00:14'!
storeMethodDeclaration: methodNode 
	| methodName |
	methodName := methodNode selector.
	methodName = #main
		ifTrue: [source nextPutAll: 'task']
		ifFalse: [source nextPutAll: 'safecall'; space; nextPutAll: (self returnTypeOf: methodNode)].
	source space; nextPutAll: (methodName asIdentifier: false); nextPut: $(.
	methodNode arguments do: [:arg | source nextPutAll: (variableInfo typeOf: arg name in: methodName); space; nextPutAll: arg name] separatedBy: [source nextPut: $,; space].
	source nextPut: $)! !

!NxcTranslator methodsFor: 'as yet unclassified' stamp: 'Richo 1/3/2011 18:46'!
storeTask: aBlockNode named: aString 
	source nextPutAll: 'task';
		 space;
		 nextPutAll: aString;
		 nextPutAll: '()';
		 cr;
		 nextPut: ${; cr.
	self visitNode: aBlockNode.
	source nextPut: $};
		 cr! !

!NxcTranslator methodsFor: 'translating' stamp: 'Richo 1/3/2011 18:46'!
translate: aClass asProgramNamed: programName 
	| methods |
	source := WriteStream on: ''.
	source nextPutAll: '/*';
		 space;
		 nextPutAll: programName;
		 space;
		 nextPutAll: '*/';
		 cr;
		 cr.
	self extractVariablesFrom: aClass.
	variableInfo instVars
		associationsDo: [:association | 
			| name type |
			name := association key.
			type := association value.
			source nextPutAll: type;
				 space;
				 nextPutAll: name;
				 nextPut: $;;
				 cr].
	source cr.
	methods := aClass selectors
				collect: [:each | aClass parseTreeFor: each].
	methods
		do: [:each | 
			self storeMethodDeclaration: each.
			source nextPut: $;;
				 cr].
	tasks := NxcTaskExtractor new extractTasksFrom: aClass.
	tasks
		do: [:association | self storeTask: association key named: association value] separatedBy: [source cr].
	source cr.
	methods
		do: [:each | self visitNode: each]
		separatedBy: [source cr; cr].
	^ NxcProgram name: programName source: source! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 03:25'!
visitArguments: aNodeCollection 
	"Do nothing"! !
NxcPrimitiveRule initialize!
